@@ -5,10 +5,12 @@ from typing import Optional
 
 log = logging.getLogger(__name__)
 
+
 def safe_filename(title: str) -> str:
     pattern = re.compile(r'[?"|:<>*/\\]', flags=re.VERBOSE)
     return pattern.sub("", str(title))
 
+
 def create_path(path: str) -> str:
     if path:
         if not os.path.isabs(path):
@@ -22,12 +24,14 @@ def create_path(path: str) -> str:
     os.makedirs(path, exist_ok=True)
     return path
 
+
 def create_tags(text: str) -> str:
     identifiers = [
-        r'is_limited_read',
-        r"is_limited_free",
+        r'is_read_for_free',
+        r"is_wait_for_free",
         r"is_already_read",
         r"is_free",
+        r"is_zero_plus",
         r"is_purchased",
     ]
     identifiers = "|".join(identifiers)
@@ -37,9 +41,23 @@ def create_tags(text: str) -> str:
     tags = regex.sub(r"episode['\1']", text.strip('"'))
     return tags
 
+
 def trunc_title(title: str) -> str:
     return re.sub(r"\((?:[^)(]|\([^)(]*\))*\)", "", title)
 
+
+def pad_string(
+    text: str,
+    length: Optional[int] = 0,
+    padding: Optional[str] = "0"
+) -> str:
+    if len(text) <= length:
+        for pad in range(length-len(text)):
+            padding += "0"
+        text = f"{padding}{text}"
+    return text
+
+
 def valid_url(url: str, level: Optional[int] = None) -> bool:
     base_url = r"(http|https)://(|www.)piccoma.com/web"
     urls = [
